library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Datapath_tb is
end entity;

architecture bhv of Datapath_tb is

  component Datapath
    port(
      rst : in std_logic;
      clk : in std_logic;

      --control signals
      alu_src_A   : in std_logic;
      alu_src_B   : in std_logic_vector( 1 downto 0 );
      -- output from the ALU control shown in Figure 5.33
      alu_control : in std_logic_vector( 2 downto 0 );
      reg_write   : in std_logic;
      reg_dst     : in std_logic;
      pc_source   : in std_logic_vector( 1 downto 0 );
      -- output from the or gate, combining PCWriteCond and PCWrite
      pc_load     : in std_logic;
      i_or_d      : in std_logic;
      mem_read    : in std_logic;
      mem_write   : in std_logic;
      mem_to_reg  : in std_logic;
      IR_write    : in std_logic;

      -- output to controller
      ins_31_26 : out std_logic_vector( 5 downto 0 );
      -- output to ALU control
      ins_5_0   : out std_logic_vector( 5 downto 0 );
      -- output to PC write logic
      alu_zero  : out std_logic;

      -- signals for testing
      testing_alu_result  : out std_logic_vector( 31 downto 0 );
      testing_mem_data    : out std_logic_vector( 31 downto 0 );
      testing_read_data_1 : out std_logic_vector( 31 downto 0 );
      testing_read_data_2 : out std_logic_vector( 31 downto 0 )

      );

  end component;

  constant C0_32 : std_logic_vector(31 downto 0)
 := "00000000000000000000000000000000";

  signal rst                                                                                                      : std_logic := '1';
  signal clk, alu_src_A, reg_write, reg_dst, pc_load, i_or_d, mem_read, mem_write, mem_to_reg, IR_write, alu_zero : std_logic;
  signal alu_src_B, pc_source                                                                                     : std_logic_vector (1 downto 0);
  signal alu_control                                                                                              : std_logic_vector (2 downto 0);
  signal ins_31_26, ins_5_0                                                                                       : std_logic_vector (5 downto 0);
  signal testing_alu_result, testing_mem_data, testing_read_data_1, testing_read_data_2                           : std_logic_vector( 31 downto 0 );

  signal pc_write_cond, pc_write : std_logic;

begin

  U_TB : Datapath port map (rst, clk, alu_src_A, alu_src_B, alu_control, reg_write, reg_dst, pc_source,
                            pc_load, i_or_d, mem_read, mem_write, mem_to_reg, IR_write, ins_31_26,
                            ins_5_0, alu_zero, testing_alu_result, testing_mem_data, testing_read_data_1,
                            testing_read_data_2);

  process
  begin
    clk <= '1';
    wait for 25 ns;
    clk <= '0';
    wait for 25 ns;
  end process;

  process

    procedure RESET is
    begin
      pc_write_cond <= '0';
      pc_write      <= '0';
      alu_src_A     <= '0';
      alu_src_B     <= "00";
      alu_control   <= "000";
      reg_write     <= '0';
      reg_dst       <= '0';
      pc_source     <= "00";
      i_or_d        <= '0';
      mem_read      <= '0';
      mem_write     <= '0';
      mem_to_reg    <= '0';
      IR_write      <= '0';
    end RESET;

    procedure STATE_0 is
    begin
      mem_read    <= '1';
      alu_src_A   <= '0';
      i_or_d      <= '0';
      IR_write    <= '1';
      alu_src_B   <= "01";
      alu_control <= "010";
      pc_write    <= '1';
      pc_source   <= "00";
      reg_write   <= '0';
    end STATE_0;

    procedure STATE_1 is
    begin	 							   
      alu_src_B <= "11";
      pc_write  <= '0';
      alu_src_A <= '0';
      alu_control <= "010";
    end STATE_1;

    procedure STATE_2 is
    begin	 									
      alu_src_A   <= '1';
      alu_src_B   <= "10";
      alu_control <= "010";
    end STATE_2;

    procedure STATE_3 is
    begin
      i_or_d   <= '1';
      mem_read <= '1';
    end STATE_3;

    procedure STATE_4 is
    begin
      reg_write  <= '1';
      mem_to_reg <= '1';
      reg_dst    <= '0';
    end STATE_4;

    procedure STATE_5 is
    begin
      i_or_d    <= '1';
      mem_write <= '1';
    end STATE_5;

    procedure STATE_6 is
    begin
      alu_src_A <= '1';
      alu_src_B <= "00";
    end STATE_6;

    procedure STATE_7 is
    begin
      reg_dst    <= '1';
      reg_write  <= '1';
      mem_to_reg <= '0';
    end STATE_7;

    procedure STATE_8 is
    begin
      alu_src_A     <= '1';
      alu_src_B     <= "00";
      alu_control   <= "110";
      pc_source     <= "01";
      pc_write_cond <= '1';
    end STATE_8;

  begin
    rst <= '1';
    RESET;
    wait for 100 ns;

--------------------------------------------------------------------------------------
    -- lw r10, 128(r0)
    -- state 0 : instruction fetch
    rst <= '0';
    wait until clk'event and clk = '1';

    RESET;
    STATE_0;				  
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "10001100000010100000000010000000") report "INS 1 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 1 : instruction decode/register fetch
    RESET;
    STATE_1;	 			
    wait until clk'event and clk = '1';
    assert (ins_31_26 = conv_std_logic_vector (35, 6) ) report "INS 1            : Incorrect opcode" severity warning;
    assert (testing_read_data_1 = C0_32 ) report "INS 1                          : Incorrect register output" severity warning;
--------------------------------------------------------------------------------------
    -- state 2 : memory address computation
    RESET;
    STATE_2;  				
    wait until clk'event and clk = '1';
    assert (testing_alu_result = conv_std_logic_vector (128, 32) ) report "INS 1 : Incorrect alu output" severity warning;
--------------------------------------------------------------------------------------
    -- state 3 : memory access
    RESET;
    STATE_3;   					
    wait until clk'event and clk = '1';
    assert (testing_mem_data = conv_std_logic_vector (7, 32) ) report "INS 1     : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 4 : write back        
    -- wait for output of mdr to be stable (requires an extra clock cycle)
    RESET;
    STATE_4;
    wait until clk'event and clk = '1';
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
    -- 8 add r12, r10, r10
    -- state 0																		  
    RESET;
    STATE_0;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "00000001010010100110000000100000") report "INS 2 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
-- state 1 : instruction decode/register fetch										  
    RESET;
    STATE_1;
    wait until clk'event and clk = '1';
    assert (ins_31_26 = "000000" ) report "INS 2                                 : Incorrect opcode" severity warning;
    assert (testing_read_data_1 = conv_std_logic_vector (7, 32) ) report "INS 2  : Incorrect register output 1" severity warning;
    assert (testing_read_data_2 = conv_std_logic_vector (7, 32) ) report "INS 2  : Incorrect register output 2" severity warning;
--------------------------------------------------------------------------------------
-- state 6 : execution 																 
    RESET;
    STATE_6;
    alu_control <= "010";
    wait until clk'event and clk = '1';
    assert (testing_alu_result = conv_std_logic_vector (14, 32) ) report "INS 2  : Incorrect alu output" severity warning;
--------------------------------------------------------------------------------------
    -- state 7 : r-type completetion													  
	RESET;
    STATE_7;
    wait until clk'event and clk = '1';
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
    -- and r11, r12, r10
    -- state 0																			
	RESET;
    STATE_0;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "00000001100010100101100000100100") report "INS 3 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 1 : instruction decode/register fetch										
	RESET;
    STATE_1;
    wait until clk'event and clk = '1';
    assert (ins_31_26 = "000000" ) report "INS 3                                 : Incorrect opcode" severity warning;
    assert (testing_read_data_1 = conv_std_logic_vector (14, 32) ) report "INS 3 : Incorrect register output 1" severity warning;
    assert (testing_read_data_2 = conv_std_logic_vector (7, 32) ) report "INS 3  : Incorrect register output 2" severity warning;
--------------------------------------------------------------------------------------
    -- state 6 : execution																
	RESET;
    STATE_6;
    alu_control <= "000";
    wait until clk'event and clk = '1';
    assert (testing_alu_result = conv_std_logic_vector (6, 32) ) report "INS 3   : Incorrect alu output" severity warning;
--------------------------------------------------------------------------------------
    -- state 7 : r-type completetion												 
	RESET;
    STATE_7;
    wait until clk'event and clk = '1';
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
    -- sw r11, 132(r0)
    -- state 0 : instruction fetch													
	RESET;
    STATE_0;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "10101100000010110000000010000100") report "INS 4 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 1 : instruction decode/register fetch									   
	RESET;
    STATE_1;
    wait until clk'event and clk = '1';
    assert (ins_31_26 = conv_std_logic_vector (43, 6) ) report "INS 4            : Incorrect opcode" severity warning;
    assert (testing_read_data_1 = C0_32 ) report "INS 4                          : Incorrect register output" severity warning;
--------------------------------------------------------------------------------------
    -- state 2 : memory address computation												 
	RESET;
    STATE_2;
    wait until clk'event and clk = '1';
    assert (testing_alu_result = conv_std_logic_vector (132, 32) ) report "INS 4 : Incorrect alu output" severity warning;
--------------------------------------------------------------------------------------
    -- state 5 : memory access														   
	RESET;
    STATE_5;
    wait until clk'event and clk = '1';
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
    -- lw r13, 132(r0)
    -- state 0 : instruction fetch         										   
	RESET;
    STATE_0;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "10001100000011010000000010000100") report "INS 5 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 1 : instruction decode/register fetch								 
	RESET;
    STATE_1;
    wait until clk'event and clk = '1';
    assert (ins_31_26 = conv_std_logic_vector (35, 6) ) report "INS 5            : Incorrect opcode" severity warning;
    assert (testing_read_data_1 = C0_32 ) report "INS 5                          : Incorrect register output" severity warning;
--------------------------------------------------------------------------------------
    -- state 2 : memory address computation
    RESET;
    STATE_2;
    wait until clk'event and clk = '1';
    assert (testing_alu_result = conv_std_logic_vector (132, 32) ) report "INS 5 : Incorrect alu output" severity warning;
--------------------------------------------------------------------------------------
    -- state 3 : memory access
    RESET;
    STATE_3;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = conv_std_logic_vector (6, 32) ) report "INS 5     : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 4 : write back        
    -- wait for output of mdr to be stable (requires an extra clock cycle)
    RESET;
    STATE_4;
    wait until clk'event and clk = '1';

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
    -- beq r11, r11, -6
    -- state 0 : instruction fetch         
    RESET;
    STATE_0;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "00010001011010111111111111111010") report "INS 6 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 1 : instruction decode/register fetch
    RESET;
    STATE_1;
    wait until clk'event and clk = '1';
    assert (ins_31_26 = conv_std_logic_vector (4, 6) ) report "INS 6             : Incorrect opcode" severity warning;
    assert (testing_read_data_1 = conv_std_logic_vector (6, 32) ) report "INS 6  : Incorrect register output 1" severity warning;
    assert (testing_read_data_2 = conv_std_logic_vector (6, 32) ) report "INS 6  : Incorrect register output 2" severity warning;
--------------------------------------------------------------------------------------
    -- state 8 : branch completion
    RESET;
    STATE_8;
    wait until clk'event and clk = '1';

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
    -- lw r10, 128(r0)
    -- state 0 : instruction fetch
    RESET;
    STATE_0;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = "10001100000010100000000010000000") report "INS 7 : Incorrect memory data" severity warning;
--------------------------------------------------------------------------------------
    -- state 1 : instruction decode/register fetch
    RESET;
    STATE_1;
    wait until clk'event and clk = '1';
    assert (ins_31_26 = conv_std_logic_vector (35, 6) ) report "INS 7            : Incorrect opcode" severity WARNING;
    assert (testing_read_data_1 = C0_32 ) report "INS 7                          : Incorrect register output" severity WARNING;
--------------------------------------------------------------------------------------
    -- state 2 : memory address computation
    RESET;
    STATE_2;
    wait until clk'event and clk = '1';
    assert (testing_alu_result = conv_std_logic_vector (128, 32) ) report "INS 7 : Incorrect alu output" severity WARNING;
--------------------------------------------------------------------------------------
    -- state 3 : memory access
    RESET;
    STATE_3;
    wait until clk'event and clk = '1';
    assert (testing_mem_data = conv_std_logic_vector (7, 32) ) report "INS 7: Incorrect memory data" severity WARNING;
--------------------------------------------------------------------------------------
    -- state 4 : write back	   
    -- wait for output of mdr to be stable (requires an extra clock cycle)
    RESET;
    STATE_4;
    wait until clk'event and clk = '1';

    wait;
  end process;
  
  pc_load <= (pc_write_cond and alu_zero) or pc_write;
  
end bhv;


